/* rotatecubeGLUT.c
 * Application affichant un cube tournant sur lui-mme.
 * Touches:
 *	ESC - Quitter
 * 	1 - Activation/Arrt de l'antialiasing
 *	2 - Activation/Arrt du mode filaire
 *
 * Auteur: Vouriot Thierry
 * Web: http://thierry.vouriot.free.fr
 * Email: thierry.vouriot@free.fr
 * Date: 4 septembre 2002
 */

#include <GL/glut.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/* Caractristiques de la fentre et de l'cran */
#define TITLE "RotateCubeGLUT (Vouriot Thierry)"	// titre du programme
#define WINDOW_WIDTH 350	// largeur de la fentre
#define WINDOW_HEIGHT 350	// largeur de la fentre
#define	WINDOW_POS_X 100	// position x de la fentre
#define WINDOW_POS_Y 100	// position y de la fentre
#define FULLSCREEN_WIDTH 640	// largeur en mode plein cran
#define FULLSCREEN_HEIGHT 480	// hauteur en mode plein cran
#define FULLSCREEN_DEPTH 16	// bit par pixels en mode plen cran

/* Dfinitions pour Windows */
#ifdef _WIN32
#include <windows.h>
#pragma comment( lib, "opengl32.lib" )			// recherche de OpenGL32.lib pendant l'dition de liens
#pragma comment( lib, "glu32.lib" )			// recherche de Glu32.lib pendant l'dition de liens
#pragma comment( lib, "glut32.lib" )			// recherche de Glut32.lib pendant l'dition de liens
#endif

/* Dfinition des boolens pour C */
#ifndef bool
#define bool unsigned char
#define true 1 == 1
#define false !true
#endif

#define ESC 27			// code ascii de la touche escape

/* Fullscreen */
static bool g_fullscreen = false;

/* Variable contenant le temps au moment de l'affichage de la
 * dernire image, cet variable permet de baser les
 * mouvements sur le temps et non pas sur la vitesse d'affichage
 * de votre ordinateur */
static int g_timeAtLastFrame = 0;

/* Variables pour la rotation du cube */
float rotX = 0.0f, rotZ = 0.0, rotY = 0.0f;	

/* Prototypes */
void init(void);
void askFullscreen(void);
void glutDisplay(void);
void glutIdle(void);
void glutReshape(int w, int h);
void glutKeyboard(unsigned char key, int x, int y);
void terminateApplication(void);
GLvoid DrawCube(float x, float y, float z, float radius);

/* Main */
int main(int argc, char ** argv)
{	
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGB);
	#ifdef _WIN32
	askFullscreen();
	#endif
	if (!g_fullscreen) {
		glutInitWindowSize(WINDOW_WIDTH, WINDOW_HEIGHT);
		glutInitWindowPosition(WINDOW_POS_X, WINDOW_POS_Y);
		glutCreateWindow(TITLE);
	}	
	init();
	glutDisplayFunc(glutDisplay);
	glutIdleFunc(glutIdle);
	glutReshapeFunc(glutReshape);
	glutKeyboardFunc(glutKeyboard);
	glutMainLoop();
	return 0;
}

/* Permet de passer en mode plein ecran ou en mode fenetre
 * Utilise GLUT_GAME_MODE disponible uniquement sous Windows
 */
#ifdef _WIN32
void askFullscreen(void)
{
	char gameMode[14];
	
	/* Demande  l'utilisateur s'il veut le mode plein cran */
	if (!g_fullscreen)
	g_fullscreen = (MessageBox(NULL, "Click Yes to go to full screen (Recommended)", "Start FullScreen?",
						MB_ICONQUESTION | MB_YESNO) == IDYES);
	
	
	if (g_fullscreen) {	// game mode (fullscreen)
		sprintf(gameMode, "%dx%d:%d", FULLSCREEN_WIDTH, FULLSCREEN_HEIGHT, FULLSCREEN_DEPTH);
		glutGameModeString(gameMode);
		// teste si le game mode est possible
		if (glutGameModeGet(GLUT_GAME_MODE_POSSIBLE))
		{
			glutEnterGameMode();
			//glutSetCursor(GLUT_CURSOR_NONE);
		}
		else {
			g_fullscreen = false;
		}
	} // mode fentr
}
#endif

/* Initialisation */
void init(void)
{
	/* Affiche des informations sur OpenGL */
	char *exten;
	exten = (char *) glGetString(GL_VENDOR);
	printf("%s\n",exten);
	exten = (char *) glGetString(GL_RENDERER);
	printf("%s\n",exten);
	exten = (char *) glGetString(GL_VERSION);
	printf("%s\n",exten);
	
	printf("\nTouches :\n");
	printf("\tESC - Quitter\n");
	printf("\t1 - Activation/Arret de l'antialiasing\n");
	printf("\t2 - Activation/Arret du mode filaire\n");
	
	g_timeAtLastFrame = glutGet(GLUT_ELAPSED_TIME);
	
	/* Initialisation d'OpenGL */
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);	
	glShadeModel(GL_SMOOTH);
	glEnable(GL_DEPTH_TEST);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
}
		
/* Affichage et animation */
void glutDisplay(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	
	// 	  Position      View	   Up Vector
	gluLookAt(0, 0, 6,     0, 0, 0,     0, 1, 0);

	// Rotation
	glRotatef(rotX, 1.0f, 0.0f, 0.0f);
	glRotatef(rotY, 0.0f, 1.0f, 0.0f);
	glRotatef(rotZ, 0.0f, 0.0f, 1.0f);

	DrawCube(-1, -1, -1, 2);
	
	glFlush();
	glutSwapBuffers();
}

/* Fonction excut lorsque le programme n'a plus rien  faire */
void glutIdle(void)
{
	int currentTime = glutGet(GLUT_ELAPSED_TIME);
	int frameInterval = currentTime - g_timeAtLastFrame; // Temps coul entre l'affichage de 2 images
	g_timeAtLastFrame = currentTime;
	
	// On incrmente les angles de rotation
	rotX += (float)frameInterval / 15;
	rotY += (float)frameInterval / 15;
	rotZ += (float)frameInterval / 10;
	
	if (rotX >= 360.0f)
		rotX = 0.0f;
	if (rotY >= 360.0f)
		rotY = 0.0f;
	if (rotZ >= 360.0f)
		rotZ = 0.0f;

	glutPostRedisplay();
}

/* Redimensionnement de la fentre */
void glutReshape(int w, int h)
{
	if (h == 0) h = 1;
	glViewport(0, 0, (GLsizei)w, (GLsizei)h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0f,(GLfloat)w/(GLfloat)h, 0.1f, 20.0f);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

/* Gestion du clavier */
void glutKeyboard(unsigned char key, int x, int y)
{
	static bool   antialias = false;    // antialiasing
  	static GLenum polyMode  = GL_FILL;  // polygone mode
	
	switch(key) {
		case ESC:	// touche escape
			terminateApplication();
			break;
		case '1': antialias = !antialias;
			// activation de l'antialiasing
          		if (antialias)
          		{
            			glEnable(GL_POINT_SMOOTH);
            			glEnable(GL_LINE_SMOOTH);
            			glEnable(GL_POLYGON_SMOOTH);
            			glEnable(GL_BLEND);
          		}
          		// arrt de l'antialiasing
          		else
          		{
            			glDisable(GL_POINT_SMOOTH);
            			glDisable(GL_LINE_SMOOTH);
            			glDisable(GL_POLYGON_SMOOTH);
            			glDisable(GL_BLEND);
          		}
				break;
		case '2':
			// passage du mode filaire au mode plein ou inversement
			if (polyMode == GL_FILL) polyMode = GL_LINE; else polyMode = GL_FILL;
			glPolygonMode(GL_FRONT_AND_BACK, polyMode);
			break;	
	}
}

/* Termine l'application */
void terminateApplication(void)
{
	#ifdef _WIN32
	if (g_fullscreen) glutLeaveGameMode();
	#endif
	printf("\nAuteur: Thierry Vouriot\nhttp://thierry.vouriot.free.fr\n");
	exit(1);
}

/* Dessine un cube multicolor de taille radius  la postion x, y, z */
GLvoid DrawCube(float x, float y, float z, float radius)
{
	glPushMatrix();
		glTranslatef(x, y, z);
		glBegin(GL_QUADS);
		
			// face arrire
			glColor3ub(0, 255, 255);   
			glVertex3f(0.0f, 0.0f, 0.0f);
			glVertex3f(0.0f, radius, 0.0f);
			glVertex3f(radius, radius, 0.0f); 
			glVertex3f(radius, 0.0f, 0.0f);

			// face avant
			glColor3ub(0, 255, 0);   
			glVertex3f(0.0f, 0.0f, radius);
			glVertex3f(0.0f, radius, radius);
			glVertex3f(radius, radius, radius); 
			glVertex3f(radius, 0.0f, radius);

			// face de dessous
			glColor3ub(255, 0, 0);   
			glVertex3f(0.0f, 0.0f, 0.0f);
			glVertex3f(0.0f, 0.0f, radius);
			glVertex3f(radius, 0.0f, radius); 
			glVertex3f(radius, 0.0f, 0.0f);
			
			// face de dessus
			glColor3ub(0, 0, 255);   
			glVertex3f(0.0f, radius, 0.0f);
			glVertex3f(0.0f, radius, radius);
			glVertex3f(radius, radius, radius); 
			glVertex3f(radius, radius, 0.0f);

			// face gauche
			glColor3ub(255, 0, 255);   
			glVertex3f(0.0f, 0.0f, 0.0f);
			glVertex3f(0.0f, 0.0f, radius);
			glVertex3f(0.0f, radius, radius); 
			glVertex3f(0.0f, radius, 0.0f);

			// face droite
			glColor3ub(255, 255, 0);   
			glVertex3f(radius, 0.0f, 0.0f);
			glVertex3f(radius, 0.0f, radius);
			glVertex3f(radius, radius, radius); 
			glVertex3f(radius, radius, 0.0f);
			glEnd();
	glPopMatrix();
}



